-- Lost Lore reference plugin by David Down
-- coding: utf-8 'ä
import "Turbine.Gameplay"
import "Vinny.Common"
import "Vinny.LostLore.LL_Data"
import "Vinny.Common.Help"

function print(text) Turbine.Shell.WriteLine("<rgb=#00FFFF>LL:</rgb> "..text) end
function printh(text) print("<rgb=#00FF00>"..text.."</rgb>") end
function printe(text) print("<rgb=#FF6040>Error: "..text.."</rgb>") end

local Coord = "^(%d+%.%d[NnSs]),(%d+%.%d[EeWw])$"
local Cloc = "(%d+%.%d[NS]), (%d+%.%d[EW])"
local Zloc = "^(.+): .+: (%d+%.%d[NS]), (%d+%.%d[EW])$"
red,yel,grn,mag = "FF0000", "FFFF00", "00FF00", "FF00FF"
local snl,enl = " <rgb=#00FF00>(", ")</rgb>"
local LLv = "Lost Lore "..Plugins["LostLore"]:GetVersion()

player = Turbine.Gameplay.LocalPlayer.GetInstance()
pname = player:GetName()
if pname:sub(1,1)=="~" then
	printe("Session Play detected.")
	return
end
plevel = player:GetLevel()
region = 0 -- no validation if 0

LL_Settings = Turbine.PluginData.Load(Turbine.DataScope.Server,"LostLore_Settings")
if type(LL_Settings) ~= "table" then
	LL_Settings = { }
    printh(LLv..", settings initialized.")
else printh(LLv..", settings loaded.") end
LL_Checks = Turbine.PluginData.Load(Turbine.DataScope.Character,"LL_Checks")
if type(LL_Checks) ~= "table" then LL_Checks = {} end
for i,n in pairs(Fix) do
	if LL_Checks[i] then
		LL_Checks[n] = LL_Checks[i]
		LL_Checks[i] = nil
		print("Renamed the Map '"..i.."' to '"..n.."'")
	end
end

import "Vinny.LostLore.LL_Window"

Plugins.LostLore.Open = function(sender,args)
	LL_window:SetVisible( true )
	LL_window:SetZOrder( 2 )
end

LL_Command = Turbine.ShellCommand()

--local sa1,sa2,sa3 = 0.19509,0.38268,0.55557
local function distance(dy,dx,head)
	local d = math.sqrt(dy*dy+dx*dx)
	if (not head) or d<0.1 then return d end
	local ix = math.floor(math.atan2(dx,dy)/math.pi*8+8.5)
	if ix>15 then ix = 0 end
	return d,ix
end

local function locV(str,neg)
    local nbr = tonumber(str:sub(1,-2))
    if neg:find(str:sub(-1)) then nbr = -nbr end
    return nbr
end

local function locC(str)
	local y,x = str:match(Cloc)
	if y and x then return y:lower()..','..x:lower() end
end

function Cfind(loc)
	for map,Atbl in pairs(Lore) do
		for area,Itbl in pairs(Atbl) do
			if Itbl.r==region or Itbl.r==0 then
				for ix,c in pairs(Itbl.p) do
					if type(c)=="table" then c = c[1] end
					if c==loc then return map,area,ix end
				end
			end
		end
	end
end

function LL_Map(ploc)
	local tbl = LL_window.tbl
	if not tbl then printe("No area selected.") return end
	local list = {}
	for ix,loc in ipairs(tbl) do
		if type(loc)=='table' then
			loc = loc[1] 
		end
		local y,x = loc:match(Coord)
		list[ix] = {y=locV(y,"Ss"), x=locV(x,"Ww")}
	end
	local xh,xl,yh,yl = -999,999,-999,999
	for ix,loc in ipairs(list) do
		if loc.x>xh then xh=loc.x end
		if loc.x<xl then xl=loc.x end
		if loc.y>yh then yh=loc.y end
		if loc.y<yl then yl=loc.y end
	end
	local dx,dy = xh-xl, yh-yl
	local cx,cy = (xh+xl)/2, (yh+yl)/2
	local w = (dx>dy and dx or dy)*1.1
	local dw = Msize/w
	for ix,loc in ipairs(list) do
		-- Add location text box
		local box = Turbine.UI.Lotro.TextBox()
		box:SetParent( LL_Mwindow )
		box:SetPosition( Msize/2-10+(loc.x-cx)*dw,Msize/2+8+(cy-loc.y)*dw )
		box:SetSize( ix>9 and 23 or 16, 15 )
		box:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter )
		box:SetFont( Turbine.UI.Lotro.Font.Verdana12 )
		box:SetSelectable( false )
		box:SetReadOnly( true )
		box:SetText( ix )
		box:SetBackColor( Turbine.UI.Color( 0, 0, 0 ) )
		if LL_Cwindow and LL_Cwindow.box and LL_Cwindow.box[ix]:IsChecked() then
			box:SetForeColor( Turbine.UI.Color( 0, 0.9, 0 ) )
		end
	end
	local reg,y,x = ploc:match(Zloc)
	if not y then return end
	if region>0 and Region[reg]~=region then printe("Not in this region.") return end
	local y1,x1 = locV(y,"Ss"), locV(x,"Ww")
	if y1<cy-dw or y1>cy+dw or x1<cx-dw or x1>cx+dw then printe("Not 0n this map.") return end
	-- create a red dot for current location
	local dot = Turbine.UI.Control()
	dot:SetParent( LL_Mwindow )
	dot:SetPosition( 58,58 )
	dot:SetPosition( Msize/2-1+(x1-cx)*dw,Msize/2+14+(cy-y1)*dw )
	dot:SetSize( 6,6 )
	dot:SetBackground( "Vinny/LostLore/Red dot2.tga" )

end

function Near(loc)
	printh("Finding collections near "..loc)
	local y0,x0,m,a,tbl = loc:match(Coord)
	y0,x0 = locV(y0,"Ss"), locV(x0,"Ww")
	local found=0
	for map,Atbl in pairs(Lore) do
		local cmap = LL_Checks[map]
		for area,Itbl in pairs(Atbl) do
			if (Itbl.r==region or Itbl.r==0) and not (cmap and (cmap[area]==true)) then
				for ix,c in pairs(Itbl.p) do
					if type(c)=="table" then c = c[1] end
					local y,x = c:match(Coord)
					if not y or not x then printe("Bad loc: "..c) end
					y,x = locV(y,"Ss"), locV(x,"Ww")
					local d = distance(y-y0,x-x0)
					if d<10 then
						found = found+1
						print(map..':'..area..', d='..(math.floor(d*10+0.5)/10))
						m = map; a = area; tbl = Itbl
						break
					end
				end
			end
		end
	end
	if found==0 then print("(None found)")
	elseif found==1 then 
		LL_window.mapMenu:SetText( m )
		LL_window.areaMenu:SetText( a )
		LL_window.checkButton:SetEnabled( true )
		LL_window.nearButton:SetEnabled( true )
		LL_window.tbl = tbl.p
		region = tbl.r
	end
end

local CD = {[0]='S','SSW','SW','WSW','W','WNW','NW','NNW',
				'N','NNE','NE','ENE','E','ESE','SE','SSE'}
function Location(args,find)
	local reg,y,x = args:match(Zloc)
	if not y then printe("No location data in instance.") return end
	if region>0 and Region[reg]~=region then printe("Not in this region.") return end
	local y1,x1 = locV(y,"Ss"), locV(x,"Ww")
	local mi, loc, desc
	if find then
		local map = LL_window.mapMenu:GetText()
		local area = LL_window.areaMenu:GetText()
		if #area<1 then return end
		print("Finding the nearest unvisited location:")
		local pages = Lore[map][area]
		local checked = LL_Checks[map]
		if checked then checked = checked[area] end
		if checked==true then printe("No locations left.") return end
		local md = 9999
		for ix,val in ipairs(pages.p) do
			if type(val)=='table' then val = val[1] end
			local y0,x0 = val:match(Coord)
			y2,x2 = locV(y0,"Ss"), locV(x0,"Ww")
			local d = distance(y2-y1,x2-x1)
			if d<md and not (checked and checked[ix]) then 
				md = d
				mi = ix
			end
		end
		LL_window.pageMenu:SetText( mi )
		loc, desc = LL_Page(pages, mi)
		LL_window.headButton:SetEnabled( true )
	else loc = LL_window.pageLoc:GetText() end
	if #loc<1 then return end
	local y0,x0 = loc:match(Coord)
	y2,x2 = locV(y0,"Ss"), locV(x0,"Ww")
	local d,h = distance(y2-y1,x2-x1,true)
	d = string.format("%.1f",d)
	if mi then loc = '#'..mi..' @ '..loc end
	print(loc.." is "..((h and d..'m '..CD[h]) or 'here'))
	if desc and find then print(desc) end
	LL_compass.box:SetText( d )
	local c = Point[h or 16]
	LL_compass.dot:SetPosition( c.x,c.y )

	if h or not LL_Cwindow or not LL_Cwindow:IsVisible() then return end
	local ix = tonumber(LL_window.pageMenu:GetText())
	desc = LL_window.pageDesc:GetText()
	if ix and not (desc and desc:sub(1,1)=='(' and not desc:find('@')) then 
		LL_Cwindow.box[ix]:SetChecked(true)
	end
end

function CheckList( done )
	printh("Check Lists that are "..(done and "done:" or "active:"))
	local found = false
	for map,tbl in pairs(LL_Checks) do
		local areas = Lore[map]
		if areas then
			for name,list in pairs(tbl) do
				local str = map..":"..name
				if list ~= true then
					if not done then
					local tot = #areas[name].p
					local cnt = 0
					for i,v in pairs(list) do
						if v then cnt = cnt+1 end
					end
					print(str.." ("..cnt.."/"..tot..")")
					found = true
					end
				elseif done then print(str); found = true end
			end
		end
	end
	if not found then print("(None found)") end
end

function SetItem(map,area,ix)
	LL_window.mapMenu:SetText( map )
	LL_window.areaMenu:SetText( area )
	LL_window.pageMenu:SetText( ix )
	if LL_Cwindow then LL_Cwindow:SetVisible( false ) end
	local cmap = LL_Checks[map]
	local done = cmap and (cmap[area]==true)
	area = Lore[map][area]
	region = area.r
	LL_window.tbl = area.p
	LL_Page(area, ix)
	LL_window.checkButton:SetEnabled( not done )
	LL_window.checkBox:SetEnabled( true )
	LL_window.checkBox:SetChecked( done )
	LL_window.pageType:SetText( LL_Name(area) )
	LL_window.nearButton:SetEnabled( true )
	LL_window.headButton:SetEnabled( true )
end

-- Main command processing
function LL_Command:Execute( cmd,args,str )
	if cmd=="ll?" then
		Vinny.Common.Help(help,"??")
		return
	end
	if args=="help" or args=="?" then
		Vinny.Common.Help(help, cmd=="ll" and args or cmd )
		return
	end
	if cmd=="llw" then
		if args=='c' then 
			local v = LL_compass:IsVisible()
			LL_compass:SetVisible( not v )
			print("Toggled Compass window")
		elseif args=='cc' then 
			local v = LL_compass.slot:IsVisible()
			LL_compass.slot:SetVisible( not v )
			print("Toggled Compass click")
		else LL_window:SetVisible( true ) end
		return
	end
	if cmd=="lln" then
		Location(args,true)
		return
	end
	if cmd=="llh" then
		Location(args)
		if LL_compass.slot:IsShiftKeyDown() then 
			LL_compass:SetVisible( true )
			LL_compass.slot:SetVisible( true )
		end
		return
	end
	if cmd=="llm" then
		LL_Mwindow = LL_MWindow()
		LL_Mwindow:SetVisible( true )
		LL_Map(args)
		return
	end
	if cmd=="lln" then
		printh("Finding matching names for '"..args.."':")
		args = args:lower()
		local found
		for map,Atbl in pairs(Lore) do
			for area in pairs(Atbl) do
				if area:lower():find(args) then
					found = true
					print(map..':'..area)
				end
			end
		end
		if not found then print("(None found)") end
		return
	end
	if cmd=="llc" then
		local loc = locC(args)
		if loc then
			print(args..' = "'..loc..'",')
		else printe("No coordinates in "..args) end
		return
	end
	if cmd=="llf" then
		local loc = locC(args)
		if loc then
			local reg = args:match(Zloc)
			region = Region[reg]
			if LL_window:IsShiftKeyDown() then
				Near(loc)
				return
			end
			local map,area,ix = Cfind(loc)
			if map then
				print(map..':'..area..'#'..ix..' = '..loc)
				SetItem(map,area,ix)
			else printe("No exact match found for "..loc) end
		else printe("No coordinates in "..args) end
		return
	end
	if args=="show" then
		LL_window:SetVisible( true )
		return
	end
	if args=="maps" then
		printh("Map names found:")
		for i,name in pairs(Maps) do
			print('  '..name)
		end
		return
	end
	if args=="done" then
		CheckList( true )
		return
	end
	if args=="part" then
		CheckList( false )
		return
	end
	if args=="quest" or args=="deed" then
		printh("Checking for available "..args..'s:')
		local t,found = args=="deed" and 'D' or 'Q'
		for map,mt in pairs(Lore) do
			local ch = LL_Checks[map]
			for area,at in pairs(mt) do
				if at.l<=plevel and at.t==t and not (ch and ch[area]) then
					found = true
					print(map..':'..area..', level='..at.l)
				end
			end
		end
		if not found then print("None found") end
		return
	end
	local m = Lore[args]
	if m then
		printh("Object categories in "..args)
		for name in pairs(m) do
			print('  '..name)
		end
		return
	end
    Vinny.Common.Help(help,args)
end

Turbine.Shell.AddCommand( "ll;llc;llf;llh;llm;lln;llw;ll?",LL_Command )

Plugins.LostLore.Unload = function(sender,args)
--    Turbine.PluginData.Save(Turbine.DataScope.Server,"LostLore_Settings",LL_Settings)
    Turbine.PluginData.Save(Turbine.DataScope.Character,"LL_Checks",LL_Checks)
    print(LLv..", settings saved.")
end

-- Options panel
import "Vinny.Common.Options"
OP = Vinny.Common.Options_Init(print,LL_Settings,LL_window,"LostLore_Settings",LL_Dwindow)

-- Help text
help = {
	pre = "LL",
	help = "Enter '/ll ?' for possible arguments",
	['?'] = "Unknown, enter '/ll ?'",
	arg = {
		deed = "Show availabe deeds.",
		quest = "Show availabe quests.",
		done = "Show finished check lists.",
		part = "Show active check lists.",
		maps = "List known map names.",
		show = "Open Lost Lore window.",
		["<map>"] = "Get categories in the map area <map>",
	},
	cmd = {
		llm = "Open/update the Lore Map window",
		lln = "Find names matching <string>",
		llw = {
			[" "] = "Open Lost Lore window.",
			c = "Toggle Compass window visibility.",
			cc = "Toggle Compass window click.",
		},
	},
}
