-- Lost Lore window handler
-- coding: utf-8 'ä
import "Turbine.UI.Lotro"
import "Vinny.Common.DropMenu"

local labelFont = Turbine.UI.Lotro.Font.TrajanPro14
local textFont = Turbine.UI.Lotro.Font.TrajanPro16
local foreColor = Turbine.UI.Color( 0.9, 0.9, 0 )
local textColor = Turbine.UI.Color( 1, 1, 0.8 )
local backColor = Turbine.UI.Color( 0, 0, 0.1 )
local Alias = Turbine.UI.Lotro.ShortcutType.Alias
local Button = Turbine.UI.Lotro.Button
local CheckBox = Turbine.UI.Lotro.CheckBox
local DropMenu = Vinny.Common.DropMenu
local Label = Turbine.UI.Lotro.GoldButton
local TextBox = Turbine.UI.Lotro.TextBox
local Left = Turbine.UI.ContentAlignment.MiddleLeft
local Center = Turbine.UI.ContentAlignment.MiddleCenter
local Right = Turbine.UI.ContentAlignment.MiddleRight
local Coord = "^(%d+%.%d[ns]),(%d+%.%d[ew])$"
LL_checked = {}

LL_Window = class( Turbine.UI.Lotro.Window )

function LL_Name(area)
	local name = area.d
	if name:find(' ') then 
		if name:sub(-1)==' ' then name = name:sub(1,-2) end
		return name
	end
	return name.." page"
end

function LL_Page(area,ix)
	local desc
	local page = area.p[tonumber(ix)]
	if type(page)=='table' then
		desc = page[2]
		if page[3] then desc = '('..desc..'): '..page[3] end
		page = page[1]
	end
	LL_window.pageLoc:SetText( page )
	LL_window.pageDesc:SetText( desc or '' )
	return page,desc
end

function LL_rewards(str)
	local txt = "Rewards: "..Reward[str:sub(1,1)]
	for i = 2,#str do
		txt = txt..', '..Reward[str:sub(i,i)]
	end
	print(txt)
end

function LL_List(map,area,neg)
	if map=="" then printe("Select a map.") return end
	if area=="" then printe("Select an area.") return end
	local pages = Lore[map][area]
	printh(LL_Name(pages).."s in "..map..":"..area)
	local list,yx = {}, {}
	for ix,loc in ipairs(pages.p) do
		local tb = {i=ix, l=loc}
		if type(loc)=='table' then
			tb.l = loc[1] 
			if loc[3] then tb.n = ' ('..loc[2]..'): '..loc[3] 
			else tb.n = ' '..loc[2] end
		end
		local y,x = tb.l:match(Coord)
		local str = neg=='s' and y or x
		local nbr = tonumber(str:sub(1,-2))
		if neg==str:sub(-1) then nbr = -nbr end
		tb.v = nbr
		table.insert(list,tb)
	end
	table.sort(list,function (a,b) return a.v > b.v end)
	for ix,loc in ipairs(list) do
		print('#'..loc.i..(loc.i<10 and ' ' or '').." @ "..loc.l..(loc.n or ''))
	end
end

function LL_Window:AddField(control, text, pos, size)
	local field = control()
	field:SetParent( self )
	if text then field:SetText( text ) end
	field:SetPosition( pos.x,pos.y )
	field:SetSize( size.x,size.y )
	if control==Button or control==DropMenu or not text then return field end
	field:SetBackColor( backColor )
	if control==TextBox then 
		field:SetReadOnly( true )
		field:SetForeColor( textColor )
	else field:SetForeColor( foreColor ) end
	field:SetFont( control==TextBox and textFont or labelFont )
	field:SetTextAlignment( (control==TextBox or control==DropMenu) and Left or 
					control==Label and Right or Center )
	return field
end

function LL_Click(Item,button)
	Item.Click = function( sender,args )
		local selected = sender:GetText()
		button:SetText(selected)
		print( "Selected "..selected )
		if button.Action then
			button:Action(selected)
		end
	end
end

function LL_Window:BuildMenu(list,button)
	local Menu = Turbine.UI.ContextMenu()
	local menuItems,Item = Menu:GetItems()
	local selected = button:GetText()
	for i,name in ipairs(list) do
        Item = Turbine.UI.MenuItem( name, true, name==selected )
		LL_Click(Item,button)
        menuItems:Add( Item )
	end
	return Menu
end

function LL_Window:Constructor()
	Turbine.UI.Lotro.Window.Constructor( self )
	self:SetText( "Lost Lore" )
	self:SetSize( 300,285 )

	-- Position the window near the top and to left of center of the screen.
--	self:SetBackColor( backColor )
	local pos = LL_Settings.pos1 or 
				{ x=Turbine.UI.Display.GetWidth()/5, y=self:GetHeight()/2 }
	self:SetPosition( pos.x, pos.y )

	-- Map label and menu
	self:AddField(Label, "Map:", {x=20,y=45}, {x=40,y=14} )
	self.mapMenu = self:AddField(DropMenu, "", {x=65,y=44}, {x=200,y=20} )
	local action = function()
		self.areaMenu:SetText( "" )
		self.pageMenu:SetText( "" )
		self.checkButton:SetEnabled( false )
		if LL_Cwindow then LL_Cwindow:SetVisible(false) end
		LL_Mwindow:SetVisible(false)
		self.checkBox:SetEnabled( false )
		self.checkBox:SetChecked( false )
		self.pageType:SetText( "" )
		self.nearButton:SetEnabled( false )
		self.pageLoc:SetText( "" )
		self.headButton:SetEnabled( false )
		self.pageDesc:SetText( "" )
	end
	self.mapMenu.Menu.Click = function() self.mapMenu:BuildMenu(Maps,action) end

	-- Area label and menu
	self:AddField(Label, "Area:", {x=20,y=73}, {x=40,y=14} )
	self.areaMenu = self:AddField(DropMenu, "", {x=65,y=72}, {x=200,y=20} )
	local action = function(args) 
        local map = Lore[self.mapMenu:GetText()]
		local area = map[self.areaMenu:GetText()]
		self.pageType:SetText( LL_Name(area) )
		if LL_Cwindow then LL_Cwindow:SetVisible(false) end
		LL_Mwindow:SetVisible(false)
		self.checkBox:SetEnabled( true )
		local LL_map = LL_Checks[self.mapMenu:GetText()]
		if LL_map and (LL_map[self.areaMenu:GetText()]==true) then
			self.checkBox:SetChecked( true )
		else 
			self.checkButton:SetEnabled( true )
			self.checkBox:SetChecked( false )
		end
		self.pageMenu:SetText( "" )
		self.nearButton:SetEnabled( true )
		self.pageLoc:SetText( "" )
		self.headButton:SetEnabled( false )
		self.pageDesc:SetText( "" )
		self.tbl = area.p
		region = area.r
		local lvl,t = area.l, area.t
		local act = t and Atype[t] or "action"
		if lvl then 
			local txt = "This is a level "..lvl..' '..act..'.'
			if lvl>plevel then txt = "<rgb=#FFFF00>Warning: "..txt.."</rgb>" end
			print(txt) 
		end
		if area.c then print("Log: "..area.c) end
		if area.R then LL_rewards(area.R) end
		if area.n then printh("Important Notice:\n"..area.n) end
	end
	self.areaMenu.Menu.Click = function()
        local map = self.mapMenu:GetText()
		if map~="" then
			local Loc_list = {}
			for name,t in pairs(Lore[map]) do
				table.insert(Loc_list,name)
			end
			table.sort(Loc_list)
			self.areaMenu:BuildMenu(Loc_list,action)
		else printe("Select a map.") end
	end

	-- Item label and menu
	self:AddField(Label, "Item:", {x=20,y=101}, {x=40,y=14} )
	self.pageMenu = self:AddField(DropMenu, "", {x=65,y=100}, {x=55,y=20} )
	local action = function(args) 
		local pages = Lore[self.mapMenu:GetText()][self.areaMenu:GetText()]
		local loc,desc = LL_Page(pages, self.pageMenu:GetText())
		self.pageLoc:SetText( loc )
		print( LL_Name(pages).." @ "..loc )
		if desc then print(desc) end
		self.headButton:SetEnabled( true )
	end
	self.pageMenu.Menu.Click = function()
        local map = self.mapMenu:GetText()
		if map=="" then printe("Select a map.") return end
        local area = self.areaMenu:GetText()
		if area~="" then
			local Loc_list = {}
			for name,t in pairs(Lore[map][area].p) do
				table.insert(Loc_list,name)
			end
			table.sort(Loc_list)
			self.pageMenu:BuildMenu(Loc_list,action)
		else printe("Select an area.") end
	end

	-- Create an Check List button
	self.checkButton = self:AddField(Button, "Check List", {x=130,y=101}, {x=85,y=20} )
	self.checkButton:SetEnabled( false )
	self.checkButton.Click = function( sender,args )
        local map = self.mapMenu:GetText()
        local area = self.areaMenu:GetText()
		local pages = Lore[map][area]
		local list = {}
		for ix,loc in ipairs(pages.p) do
			if type(loc)=='table' then
				loc = loc[1] 
			end
			list[ix] = loc
		end
		print("Opening Check List window")
		if not LL_Checks[map] then LL_Checks[map] = {} end
		local LL_map = LL_Checks[map]
		if not LL_map[area] then LL_map[area] = {} end
		LL_checked = LL_map[area]
		LL_Cwindow = LL_CWindow(list)
		LL_Cwindow:SetVisible( true )
	end

	-- Done check box
	self.checkBox = self:AddField(CheckBox, "Done", {x=225,y=103}, {x=55,y=16} )
	self.checkBox:SetEnabled( false )
	self.checkBox.CheckedChanged = function( sender,args )
		Done = sender:IsChecked()
		print((Done and "Set " or "Cleared ").."Done.")
		self.checkButton:SetEnabled(not Done)
        local map = self.mapMenu:GetText()
        local area = self.areaMenu:GetText()
		local LL_map = LL_Checks[map]
		if Done then
			if not LL_map then
				LL_Checks[map] = {}
				LL_map = LL_Checks[map]
			end
			LL_map[area] = true
			if LL_Cwindow then LL_Cwindow:SetVisible(false) end
			Turbine.PluginData.Save(Turbine.DataScope.Character,"LL_Checks",LL_Checks)
		elseif LL_map then LL_map[area] = nil end
	end

	-- Page type
	self:AddField(Label, "Type:", {x=19,y=130}, {x=40,y=14} )
	self.pageType = self:AddField(TextBox, "", {x=62,y=132}, {x=150,y=18} )
	self.pageType:SetFont( Turbine.UI.Lotro.Font.TrajanPro15 )

	-- Nearest button
	self.nearButton = self:AddField(Button, "Nearest", {x=214,y=131}, {x=66,y=15} )
	self.nearButton:SetEnabled( false )
	local slot = Turbine.UI.Lotro.Quickslot()
	slot:SetParent( self.nearButton )
    slot:SetPosition( 2,2 )
    slot:SetSize( 69,15 )
    slot:SetShortcut(Turbine.UI.Lotro.Shortcut( Alias, "/lln ;loc" ))
    slot:SetAllowDrop( false )

	-- Page location
	self:AddField(Label, "Loc:", {x=19,y=151}, {x=40,y=14} )
	self.pageLoc = self:AddField(TextBox, "", {x=62,y=153}, {x=150,y=18} )
	
	-- Heading button
	self.headButton = self:AddField(Button, "Heading", {x=213,y=152}, {x=68,y=15} )
	self.headButton:SetEnabled( false )
	self.headButton.Click = function( sender,args )
		LL_compass:SetVisible( true )
		if sender:IsShiftKeyDown() then LL_compass.slot:SetVisible( true ) end
	end
	local slot = Turbine.UI.Lotro.Quickslot()
	slot:SetParent( self.headButton )
    slot:SetPosition( 1,1 )
    slot:SetSize( 68,15 )
    slot:SetShortcut(Turbine.UI.Lotro.Shortcut( Alias, "/llh ;loc" ))
    slot:SetAllowDrop( false )

	-- Page description
	self.pageDesc = self:AddField(TextBox, "", {x=15,y=173}, {x=275,y=18} )

	-- Create an items button
	self.itemsButton = self:AddField(Button, "List Items", {x=27,y=197}, {x=88,y=20} )
	self.itemsButton.Click = function( sender,args )
        local map = self.mapMenu:GetText()
		if map=="" then printe("Select a map.") return end
        local area = self.areaMenu:GetText()
		if area=="" then printe("Select an area.") return end
		local pages = Lore[map][area]
		printh(LL_Name(pages).."s in "..map..":"..area)
		for ix,loc in ipairs(pages.p) do
			if type(loc)=='table' then
				local desc = loc[2]
				if loc[3] then desc = '('..desc..'): '..loc[3] end
				loc = loc[1]..' '..desc 
			end
			print('#'..ix.." @ "..loc)
		end
	end

	-- Create a N-S items button
	self.nsButton = self:AddField(Button, "List N-S", {x=125,y=197}, {x=70,y=20} )
	self.nsButton.Click = function( sender,args )
        local map = self.mapMenu:GetText()
        local area = self.areaMenu:GetText()
		LL_List(map,area,'s')
	end

	-- Create a E-W items button
	self.ewButton = self:AddField(Button, "List E-W", {x=205,y=197}, {x=70,y=20} )
	self.ewButton.Click = function( sender,args )
        local map = self.mapMenu:GetText()
        local area = self.areaMenu:GetText()
		LL_List(map,area,'w')
	end

	-- Save button
	self.saveButton = self:AddField(Button, "Save Item", {x=50,y=224}, {x=85,y=18} )
	self.saveButton.Click = function( sender,args )
		local item = self.pageMenu:GetText()
		if item and #item>0 then
			local map = self.mapMenu:GetText()
			local area = self.areaMenu:GetText()
			LL_Checks.Saved = {m=map, a=area, i=item}
			Turbine.PluginData.Save(Turbine.DataScope.Character,"LL_Checks",LL_Checks)
			print("Current Item saved for recall")
		else printe("No Item is selected.") end
	end

	-- Restore button
	self.restoreButton = self:AddField(Button, "Restore Item", {x=145,y=224}, {x=105,y=18} )
	self.restoreButton.Click = function( sender,args )
		local S = LL_Checks.Saved
		if S then
			local item = self.pageMenu:GetText()
			if sender:IsShiftKeyDown() and item and #item>0 then
				local map = self.mapMenu:GetText()
				local area = self.areaMenu:GetText()
				SetItem(S.m, S.a, S.i)
				LL_Checks.Saved = {m=map, a=area, i=item}
				print("Saved Item swapped.")
			else
				SetItem(S.m, S.a, S.i)
				print("Saved Item restored.")
			end
		else printe("No Item is Saved.") end
	end

	-- Find button
	self.findButton = self:AddField(Button, "Find what's here", {x=50,y=249}, {x=140,y=19} )
	local slot = Turbine.UI.Lotro.Quickslot()
	slot:SetParent( self.findButton )
    slot:SetPosition( 1,1 )
    slot:SetSize( 135,16 )
    slot:SetShortcut(Turbine.UI.Lotro.Shortcut( Alias, "/llf ;loc" ))
    slot:SetAllowDrop( false )

	-- Map button
	self.mapButton = self:AddField(Button, "Map", {x=200,y=249}, {x=50,y=19} )
	local slot = Turbine.UI.Lotro.Quickslot()
	slot:SetParent( self.mapButton )
    slot:SetPosition( 1,1 )
    slot:SetSize( 45,16 )
    slot:SetShortcut(Turbine.UI.Lotro.Shortcut( Alias, "/llm ;loc" ))
    slot:SetAllowDrop( false )

end
LL_window = LL_Window()

-- Define checklist window
LL_CWindow = class( Turbine.UI.Lotro.Window )

function LL_CWindow:AddBox(name,line)
	local box = CheckBox()
	box:SetParent( self )
	box:SetForeColor( foreColor )
	box:SetPosition( 45,15+20*line )
	box:SetSize( self:GetWidth()-30, 20 )
	box:SetTextAlignment( Left )
	box:SetFont( textFont )
	box:SetText( " #"..line..": "..name )
	box.name = name
	box.line = line
	box:SetChecked(LL_checked[line])
	box.CheckedChanged = function( sender,args )
		local v = sender:IsChecked()
		local str = v and "Set" or "Cleared"
		LL_checked[sender.line] = v or nil
		print(str.." #"..line.." found." )
	end
	return box
end

function LL_CWindow:Constructor(list)
	Turbine.UI.Lotro.Window.Constructor( self )
	self:SetText( "Check List" )

	-- Position the window near the top and to left of center of the screen.
	local pos,x,y = LL_Settings.pos2
	if not LL_Cwindow then
		if pos then x,y = pos.x,pos.y
		else x,y = Turbine.UI.Display.GetWidth()/3+100, self:GetHeight()/3 end
	else x,y = LL_Cwindow:GetPosition() end
	self:SetPosition( x,y )
	if list then
		self:SetSize( 220,50+20*#list )
		self.box = {}

		-- Create checkboxes and add them to the window.
		for ix,name in ipairs(list) do
			local box = self:AddBox(name,ix)
			self.box[ix] = box
		end

		self.PositionChanged = function( sender, args )
			local x,y = sender:GetPosition()
			LL_Dwindow:SetPosition( x,y ) -- track for pos2 setting
		end
	end
end

LL_Dwindow = LL_CWindow()

LL_Compass = class( Turbine.UI.Window )

Point = {[0]={x=58,y=116}, {x=34,y=110}, {x=17,y=98}, {x=5,y=79}, 
			{x=0,y=58}, {x=5,y=36}, {x=17,y=17}, {x=35,y=5}, 
			{x=58,y=0}, {x=81,y=6}, {x=98,y=17}, {x=111,y=35}, 
			{x=116,y=58}, {x=111,y=80}, {x=98,y=98}, {x=82,y=110},
			{x=58,y=58},
}

function LL_Compass:Constructor()
	Turbine.UI.Window.Constructor( self )
	self:SetPosition( Turbine.UI.Display.GetWidth()/2-64, Turbine.UI.Display.GetHeight()/2-64 )
	self:SetSize( 128,128 )
	self:SetBackground( "Vinny/LostLore/Compass.tga" )
--	self:SetVisible( true )
	local slot = Turbine.UI.Lotro.Quickslot()
	slot:SetParent( self )
    slot:SetPosition( 16,16 )
    slot:SetSize( 92,92 )
    slot:SetShortcut(Turbine.UI.Lotro.Shortcut( Alias, "/llh ;loc" ))
    slot:SetAllowDrop( false )
	slot:SetVisible( false )
	self.slot = slot

	-- Add distance text box
	local box = Turbine.UI.Lotro.TextBox()
	box:SetParent( self )
	box:SetPosition( 42,55 )
	box:SetSize( 42,20 )
	box:SetTextAlignment( Center )
	box:SetFont( textFont )
	box:SetSelectable( false )
	box:SetReadOnly( true )
	self.box = box

	-- creat a red dot direction pointer
	local dot = Turbine.UI.Control()
	dot:SetParent( self )
	dot:SetPosition( 58,58 )
	dot:SetSize( 12,12 )
	dot:SetBackground( "Vinny/LostLore/Red dot.tga" )
	self.dot = dot
	dot.p = 0

--[[	self.MouseClick = function( sender, args )
		local p = dot.p+1
		dot.p = p
		local c = Point[p]
		dot:SetPosition( c.x,c.y )
	end]]
end

function LL_Compass:MouseDown( args )
	if ( args.Button == Turbine.UI.MouseButton.Left ) then
		self.X = args.X
		self.Y = args.Y
		self.dragging = true
	end
end

function LL_Compass:MouseMove( args )
	local x, y = self:GetPosition()
	if ( self.dragging ) then
		self:SetPosition( x + args.X-self.X, y + args.Y-self.Y )
	end
end

function LL_Compass:MouseUp( args )
	if ( args.Button == Turbine.UI.MouseButton.Left ) then
		self.dragging = false
	end
end

LL_compass = LL_Compass()

LL_MWindow = class( Turbine.UI.Lotro.Window )
Msize = 400

function LL_MWindow:Constructor(list)
	Turbine.UI.Lotro.Window.Constructor( self )
	self:SetText( "Lore Map" )

	-- Position the window right of the main window
	local x,y = LL_window:GetPosition()
	self:SetPosition( x+305,y-70 )
	self:SetSize( Msize,Msize+14 )
end

LL_Mwindow = LL_MWindow()

-- Set Escape action
LL_window:SetWantsKeyEvents( true )
LL_window.KeyDown = function(sender, args)
	if( args.Action == Turbine.UI.Lotro.Action.Escape ) then
		LL_window:SetVisible( false )
		LL_Mwindow:SetVisible( false )
		LL_compass:SetVisible( false )
		if LL_Cwindow then LL_Cwindow:SetVisible(false) end
	end
end
